PROGRAM IsingGraf;
{This program displays a changing Ising model field of any size from 2x2 to}
{300x300.  Menus are used to select field size, temperature, initial setup,}
{and other options.}
 
USES MacIntf;     {basic Mac interface files}
     
     {$L IsingGraf.rsrc }       {link in resources}
     {$B+}                      {set bundle bit}
     {$T 'APPL' 'GENE'}         {set type and creator}
 
CONST
  MaxN = 300;         {The maximum number of rows and column allowed}
  TSize = 8;          {size of the Exp lookup table (Max DeltaE = 8)}
  RandRange = 32767;  {max size of number in lookup table}
  WaitCount = 1000;   {number of loops in wait loop}
 
 
 
 
{support constants start here}
      
      appleID = 128;     {resource IDs/menu IDs for menus}
      fileID  = 129;
      editID  = 130;
      Menu1ID = 131;
      Menu2ID = 132;
 
      appleM = 1;        {index for each menu in myMenus (array of menu handles)}
      fileM  = 2;
      editM  = 3;
      FirstM = 4;
      SecondM = 5;
 
      menuCount = 5;     {total number of menus}
 
      windowID = 128;    {resource ID for main window}
 
      undoCommand  = 1;
      {blank item 2}
      cutCommand   = 3;
      copyCommand  = 4;
      pasteCommand = 5;
      clearCommand = 6;
      {blank item 7}
      cSpecialCommand = 8; {constants identifying commands in Edit menu}
 
      M1NewSize = 1;
      M1NewTemp = 2;
      {blank item 3}
      M1YesAve = 4;
      M1NoAve = 5;
      M1NewAve = 6;
      {blank item 7}
      M1Speed = 8;         {constants identifying commands in Control menu}
      
      M2Item1 = 1;
      M2Item2 = 2;
      M2Item3 = 3;
      M2Item4 = 4;
      M2Item5 = 5;
      M2Item6 = 6;
      M2Item7 = 7;
      M2Item8 = 8;         {constants identifying commands in Setup menu}

      tempName = 'Delete Me';  {name of the stupid text file}
      
      
TYPE
  Rowtype = ARRAY[0..MaxN] of INTEGER;  {The spins}
  RowPtr = ^Rowtype;                    {pointer to a row of spins}

VAR 
  Field: ARRAY[0..MaxN] OF RowPtr;      {the field}

  N: INTEGER;         {number of rows and columns minus 1}
  
  NSteps: LongInt;    {the number of integration steps done so far}
  invNSteps: REAL;    {1/NSteps, sometimes when needed for averaging}
  FewSteps: INTEGER;  {a "Small" number of integration steps}
  
  Averaging: BOOLEAN; {TRUE if we are displaying averages}
  VeryBig: BOOLEAN;   {TRUE if the field is too big to average with LongInt}
  Speed: INTEGER;
  
  Spacing: INTEGER;
  hOffset: INTEGER;
  vOffset: INTEGER;   {parameters for display of the field}

  kToJ : REAL;        {The T* parameter (kToverJ)}
  Nm1 : INTEGER;      {number of rows and columns minus 2}
  Np1 : INTEGER;      {number of rows and columns}
  NSpins : LongInt;   {the number of spins}

  ExpLookup : ARRAY[1..TSize] OF INTEGER;  {lookup table FOR accept}
{This table is ExpLookup[i] = Round(RandRange*Exp(i/kToJ)-0.5), so if}
{ExpLookup[DeltaE] >= Random (=a random number from 0-RandRange),}
{The correct decision will be made on whether to accept or reject}

  tEAve, tESqAve, tSpinAve : REAL;  {temp. values from integration}
  EAve, ESqAve, SpinAve : REAL;     {average values}

  CurE : LongInt;         {The current total energy}
  CurESq : LongInt;       {The current total energy squared}
  CurSpin : LongInt;      {The current total spin}






{support variables start here}

    myMenus: ARRAY[1..menuCount] OF MenuHandle; {array of handles to the menus}
    dragRect: Rect;         {rectangle used to mark boundaries for dragging window}
    theChar: CHAR;          {character typed on the keyboard or keypad}
    myEvent: EventRecord;   {information about an event}
    GenWRecord: WindowRecord;{information about the main window}
    GenWind: WindowPtr;     {pointer to GenWRecord of RD window}
    whichWindow: WindowPtr; {pointer to window in which mouse button was pressed}
    hCurs: CursHandle;      {temp cursor handle}
    iBeam: Cursor;          {I-beam cursor (from resource file)}
    extended: BOOLEAN;      {TRUE if user is Shift-clicking}
    linteger: LONGINT;      {temporary longints}
    tComp: Comp;            {tempoorary computation}
    

    i,j: INTEGER;           {junk integers}
    TempRect: Rect;         {junk rectangles}
    tempStr: Str255;        {junk strings}
    SavePort: GrafPtr;      {temporary save of grafport}
    myPict: PicHandle;      {picture to be copied to scrap}

    tFile : text;           {a stupid text file}
 


{----------------------------------------------------------------------------------}
{content procedure and functions start here}

 FUNCTION Energy : LongInt;
{Returns the total energy of the Field in units of J}
{MOD operators are not used, in order to save time.  The wrap-arounds}
{are handled with separate loops.}
 VAR
   i, j : INTEGER;
   Sum : LongInt;
 BEGIN
  Sum := 0;
  FOR i := 0 TO Nm1 DO
   BEGIN
    FOR j := 0 TO Nm1 DO
     Sum := Sum - Field[i]^[ j] * (Field[i]^[ j + 1] + Field[i + 1]^[ j]);
   END;
  FOR i := 0 TO Nm1 DO
   Sum := Sum - Field[i]^[ N] * (Field[i]^[ 0] + Field[i + 1]^[ N]);
                                                           {get Row wrap-arounds}
  FOR j := 0 TO Nm1 DO
   Sum := Sum - Field[N]^[ j] * (Field[0]^[ j] + Field[N]^[ j + 1]);
                                                           {get Column wrap-arounds}
  Energy := Sum - Field[N]^[ N] * (Field[N]^[ 0] + Field[0]^[ N]);
                                                           {get very last wrap-arounds}
 END;

 FUNCTION Spin : LongInt;
{Returns the total net spin in the field}
  VAR
   i, j : INTEGER;
   Sum : LongInt;
 BEGIN
  Sum := 0;
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
    Sum := Sum + Field[i]^[ j];
  Spin := Sum;
 END;


PROCEDURE SetValues;
{Sets the current values of the energy and spin variables.}
BEGIN
 CurE := Energy;         {Start with the energy of the initial configuration}
 CurESq := CurE * CurE;
 CurSpin := Spin;        {Start with the spin of the initial configuration}
 EAve := 0;
 ESqAve := 0;
 SpinAve := 0;           {zero the accumulating averages varaibles}
 NSteps := 0;
END;

 PROCEDURE RandField;
{randomly sets all the spins to up or down, 50% in each}
  VAR
   i, j : INTEGER;
 BEGIN
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
    IF Random > 0 THEN
     Field[i]^[j] := 1
    ELSE
     Field[i]^[ j] := -1;
 SetValues;
 END;

 PROCEDURE CheckField;
{sets up the spins with a checkerboard pattern}
  VAR
   i, j : INTEGER;
 BEGIN
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
    IF ((i+j) MOD 2) > 0 THEN
     Field[i]^[ j] := 1
    ELSE
     Field[i]^[ j] := -1;
 SetValues;
 END;

 PROCEDURE IntfField;
{sets up the spins with a vertical interface}
  VAR
   i, j : INTEGER;
   No2: INTEGER;
 BEGIN
  No2 := N DIV 2;
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
    IF j >= No2 THEN
     Field[i]^[ j] := 1
    ELSE
     Field[i]^[ j] := -1;
 SetValues;
 END;

 PROCEDURE SpotField;
{sets up the spins with a big spot in the middle}
  VAR
   i, j : INTEGER;
   No2: INTEGER;
 BEGIN
  No2 := N DIV 2;
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
    IF ((i-No2)*(i-No2)+(j-No2)*(j-No2)) <= No2*No2 THEN
     Field[i]^[ j] := 1
    ELSE
     Field[i]^[ j] := -1;
 SetValues;
 END;

 PROCEDURE RingField;
{sets up the spins with a ring around the outside}
  VAR
   i, j : INTEGER;
 BEGIN
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
     Field[i]^[j] := -1;
     
  FOR i := 0 TO N DO
    BEGIN
    Field[i]^[0] := 1;
    Field[i]^[N] := 1;
    Field[0]^[i] := 1;
    Field[N]^[i] := 1;
    END;
 SetValues;
 END;
 
 PROCEDURE XField;
{sets up the spins with a big X}
  VAR
   i, j : INTEGER;
 BEGIN
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
     Field[i]^[j] := -1;
     
  FOR i := 0 TO N DO
    BEGIN
    Field[i]^[i] := 1;
    Field[i]^[N-i] := 1;
    END;
  
 SetValues;
 END;
 
 PROCEDURE LinedField;
{sets up the spins with a vertical lines}
  VAR
   i, j : INTEGER;
 BEGIN
  FOR i := 0 TO N DO
   FOR j := 0 TO N DO
     IF BitAnd(j,1) = 1 THEN
       Field[i]^[j] := 1
     ELSE
       Field[i]^[j] := -1;
     
 SetValues;
 END;
 
 PROCEDURE BField;
{sets up the spins with a big letter B}
  VAR
   i, j : INTEGER;
   No2: INTEGER;
 BEGIN
  No2 := N DIV 2;
  
  FOR i := 0 TO N DO
    FOR j := 0 TO N DO
      Field[i]^[j] := -1;
 
  FOR i := 0 TO N DO
    Field[i]^[0] := 1;
  
  FOR j := 0 TO N-1 DO
    BEGIN
    Field[0]^[j] := 1;
    Field[N]^[j] := 1;
    Field[No2]^[j] := 1;
    END;
    
  FOR i := 1 TO No2-1 DO
    Field[i]^[N] := 1;

  FOR i := No2+1 TO N-1 DO
    Field[i]^[N] := 1;

 SetValues;
 END;
 
 
 
 PROCEDURE DrawCell(r,c: INTEGER);
 {Draws out one cell, erasing the previous state}
 VAR
   x,y: INTEGER;
   cRect: Rect;
   
 BEGIN
 x := c*Spacing+hOffset;
 y := r*Spacing+vOffset;
 SetRect(cRect,x,y,x+Spacing,y+Spacing);
 IF Field[r]^[c] > 0 THEN
   PaintRect(cRect)
 ELSE
   EraseRect(cRect);
 END;

 
 PROCEDURE WalkField;
 {draws the whole field on the screan}
 VAR
   i,j: INTEGER;
   TempRect: Rect;
 BEGIN
  i := (N+1)*Spacing+hOffset;
  j := (N+1)*Spacing+vOffset;
  SetRect(TempRect,hOffset-1,vOffset-1,i+1,j+1);
  EraseRect(GenWind^.portRect); {clear area of field}
  FrameRect(TempRect); {outline area of field}
  FOR i := 0 TO N DO
    FOR j := 0 TO N DO
      DrawCell(i,j);
 END;

PROCEDURE ShowAverages;
{prints out the current average energy and spin to the screan.}
BEGIN
  setRect(tempRect,340,20,512,225);
  eraseRect(tempRect);

  StringOf(kToJ:10:4,TempStr);
  MoveTo(345,40);
  drawString(CONCAT('kT/J = ',TempStr));   

  StringOf(NSteps,TempStr);
  MoveTo(345,55);
  drawString(CONCAT('Steps = ',TempStr));   

  MoveTo(345,85);
  drawString('Current Values:');   

  StringOf(tEAve:8:4,TempStr);
  MoveTo(345,100);
  drawString(CONCAT('E = ',TempStr));   

  StringOf(tESqAve:8:4,TempStr);
  MoveTo(345,115);
  drawString(CONCAT('E^2 = ',TempStr));   
  
  StringOf(tSpinAve:8:4,TempStr);
  MoveTo(345,130);
  drawString(CONCAT('S = ',TempStr));   
  
  MoveTo(345,160);
  drawString('Averaged Values:');   

  StringOf(EAve:8:4,TempStr);
  MoveTo(345,175);
  drawString(CONCAT('<E> = ',TempStr));   

  StringOf(ESqAve:8:4,TempStr);
  MoveTo(345,190);
  drawString(CONCAT('<E^2> = ',TempStr));   
  
  StringOf(SpinAve:8:4,TempStr);
  MoveTo(345,205);
  drawString(CONCAT('<S> = ',TempStr));   
  
  StringOf(NSpins*(ESqAve-EAve*EAve)/kToJ/kToJ:8:4,TempStr);
  MoveTo(345,220);
  drawString(CONCAT('C = ',TempStr));   
END; 

PROCEDURE doUpdate;
{updates the screan in responce to a system update event}
BEGIN
  BeginUpdate(GenWind);
  WalkField;
  ShowAverages;
  EndUpdate(GenWind);
END;
  

 PROCEDURE MakeLookup (kToJ : REAL);
{Sets up the exp lookup table.  This table holds number from zero to}
{RandRange, and takes delta E as its index}
  VAR
   i : INTEGER;
 BEGIN
  FOR i := 1 TO TSize DO
   ExpLookup[i] := Round(RandRange * exp(-i / kToJ) - 0.5);
 END;


 PROCEDURE MonteCarloI (lNSteps : INTEGER;
                       VAR EAve, ESqAve, SpinAve : REAL);
{Does lNSteps of Monte Carlo integration using the weighting function}
{in ExpLookup (which determines the effective kT/J), and returns the }
{average energy, the average energy squared, the average spin, and the}
{average spin squared.}
{This version uses 32 bit integer variables to handle smallish fields.}
  VAR
   i : INTEGER;            {Index counter}
   r, c : INTEGER;         {random indicies}
   DelE : INTEGER;         {change in energy FOR flipping (r,c)}
   SumE : LongInt;         {The sum of all the total energies so far}
   SumESq : LongInt;       {The sum of all the total energies squared so far}
   SumSpin : LongInt;      {The sum of all the total spins so far}
   TempSpin : INTEGER;     {The spin of the cell being tested}
   aCount,bCount: INTEGER; {index counters for wait}
  
 BEGIN
  SumE := 0;
  SumESq := 0;
  SumSpin := 0;
  FOR i := 1 TO lNSteps DO     {loop over number of integration steps}
   BEGIN
    IF Speed <> 1 THEN
      FOR aCount := 1 TO Speed DO
        FOR bCount := 1 TO WaitCount DO {nothing} ;
   
    r := ABS(Random) MOD Np1;
    c := ABS(Random) MOD Np1;       {select a random spin}
    TempSpin := Field[r]^[ c];      {store the current state of the spin}
    DelE := 2 * TempSpin * (Field[r]^[ (c + 1) MOD Np1] + 
                            Field[r]^[ (Np1 + c - 1) MOD Np1] + 
			    Field[(r + 1) MOD Np1]^[ c] + 
			    Field[(Np1 + r - 1) MOD Np1]^[ c]);
                  {calculate the delta E for flipping this selection}
    IF DelE <= 0 THEN {accept right away}
     BEGIN
      Field[r]^[ c] := -TempSpin;  {flip the cell}
      DrawCell(r,c);
      CurE := CurE + DelE;         {Update the current total energy}
      CurESq := CurE * CurE;
      CurSpin := CurSpin - 2 * TempSpin;
     END
    ELSE IF ExpLookup[DelE] > ABS(Random) THEN {is small enough increase}
     BEGIN
      Field[r]^[ c] := -TempSpin;  {flip the cell}
      DrawCell(r,c);
      CurE := CurE + DelE;         {Update the current total energy}
      CurESq := CurE * CurE;
      CurSpin := CurSpin - 2 * TempSpin;
     END;
    SumE := SumE + CurE;         {Add the current state to the integration}
    SumESq := SumESq + CurESq;
    SumSpin := SumSpin + CurSpin;
   END;
  EAve := SumE / lNSteps / NSpins;
  ESqAve := SumESq / lNSteps / NSpins / NSpins;
  SpinAve := SumSpin / lNSteps / NSpins;   {return the average values}
 END;

 PROCEDURE MonteCarloR (lNSteps : INTEGER;
                       VAR EAve, ESqAve, SpinAve : REAL);
{Does lNSteps of Monte Carlo integration using the weighting function}
{in ExpLookup (which determines the effective kT/J), and returns the }
{average energy, the average energy squared, the average spin, and the}
{average spin squared.}
{This version uses 80 bit integer variables to handle very large fields.}
  VAR
   i : INTEGER;            {Index counter}
   r, c : INTEGER;         {random indicies}
   DelE : INTEGER;         {change in energy FOR flipping (r,c)}
   SumE : Comp;            {The sum of all the total energies so far}
   SumESq : Comp;          {The sum of all the total energies squared so far}
   SumSpin : Comp;         {The sum of all the total spins so far}
   TempSpin : INTEGER;     {The spin of the cell being tested}
   aCount,bCount: INTEGER; {index counters for wait}
  
 BEGIN
  SumE := 0;
  SumESq := 0;
  SumSpin := 0;
  FOR i := 1 TO lNSteps DO     {loop over number of integration steps}
   BEGIN
    IF Speed <> 1 THEN
      FOR aCount := 1 TO Speed DO
        FOR bCount := 1 TO WaitCount DO {nothing} ;
   
    r := ABS(Random) MOD Np1;
    c := ABS(Random) MOD Np1;         {select a random spin}
    TempSpin := Field[r]^[ c];        {store the current state of the spin}
    DelE := 2 * TempSpin * (Field[r]^[ (c + 1) MOD Np1] + 
                            Field[r]^[ (Np1 + c - 1) MOD Np1] + 
			    Field[(r + 1) MOD Np1]^[ c] + 
			    Field[(Np1 + r - 1) MOD Np1]^[ c]);
                  {calculate the delta E for flipping this selection}
    IF DelE <= 0 THEN {accept right away}
     BEGIN
      Field[r]^[ c] := -TempSpin;  {flip the cell}
      DrawCell(r,c);
      CurE := CurE + DelE;         {Update the current total energy}
      CurESq := CurE * CurE;
      CurSpin := CurSpin - 2 * TempSpin;
     END
    ELSE IF ExpLookup[DelE] > ABS(Random) THEN {is small enough increase}
     BEGIN
      Field[r]^[ c] := -TempSpin;  {flip the cell}
      DrawCell(r,c);
      CurE := CurE + DelE;         {Update the current total energy}
      CurESq := CurE * CurE;
      CurSpin := CurSpin - 2 * TempSpin;
     END;
    SumE := SumE + CurE;         {Add the current state to the integration}
    SumESq := SumESq + CurESq;
    SumSpin := SumSpin + CurSpin;
   END;
  EAve := SumE / lNSteps / NSpins;
  ESqAve := SumESq / lNSteps/ NSpins / NSpins;
  SpinAve := SumSpin / lNSteps/ NSpins;   {return the average values}
 END;

 PROCEDURE EqMonteCarlo (lNSteps : INTEGER);
{Just like MonteCarlo, except this one does not return or calculate}
{anything.  It is used for fast equilibration.}
  VAR
   i : INTEGER;            {Index counter}
   r, c : INTEGER;         {random indicies}
   DelE : INTEGER;         {change in energy FOR flipping (r,c)}
   TempSpin : INTEGER;     {The spin of the cell being tested}
   aCount,bCount: INTEGER; {index counters for wait}
  
 BEGIN
  FOR i := 1 TO lNSteps DO     {loop over number of integration steps}
   BEGIN
    IF Speed <> 1 THEN
      FOR aCount := 1 TO Speed DO
        FOR bCount := 1 TO WaitCount DO {nothing} ;
   
    r := ABS(Random) MOD Np1;
    c := ABS(Random) MOD Np1;         {select a random spin}
    TempSpin := Field[r]^[ c];        {store the current state of the spin}
    DelE := 2 * TempSpin * (Field[r]^[ (c + 1) MOD Np1] + 
                            Field[r]^[ (Np1 + c - 1) MOD Np1] + 
			    Field[(r + 1) MOD Np1]^[ c] + 
			    Field[(Np1 + r - 1) MOD Np1]^[ c]);
                  {calculate the delta E for flipping this selection}
    IF DelE <= 0 THEN {accept right away}
      BEGIN
      Field[r]^[ c] := -TempSpin;    {flip the cell}
      DrawCell(r,c);
      END
    ELSE IF ExpLookup[DelE] > ABS(Random) THEN {is small enough increase}
      BEGIN
      Field[r]^[ c] := -TempSpin;    {flip the cell}
      DrawCell(r,c);
      END;
   END;
 END;





 
{----------------------------------------------------------------------------------}
{support procedures and functions start here}

 
PROCEDURE SetUpMenus;
{ Set up menus and menu bar }
 
  VAR i: INTEGER;
 
  BEGIN
  { Read menu descriptions from resource file into memory and store handles }
  { in myMenus array }
  myMenus[appleM] := GetMenu(appleID); {read Apple menu from resource file}
  AddResMenu(myMenus[appleM],'DRVR');  {add desk accessory names to Apple menu}
  myMenus[fileM] := GetMenu(fileID);   {read File menu from resource file}
  myMenus[editM] := GetMenu(editID);   {read Edit menu from resource file}
  myMenus[FirstM] := GetMenu(Menu1ID); {read first menu from resource file}
  myMenus[SecondM] := GetMenu(Menu2ID);{read second menu from resource file}
 
  FOR i:=1 TO menuCount DO InsertMenu(myMenus[i],0);  {install menus in menu bar }
  DrawMenuBar;                                        { and draw menu bar}
  
  Averaging := TRUE;                       {default is to display averages}
  CheckItem(myMenus[FirstM],M1YesAve,TRUE);
  CheckItem(myMenus[FirstM],M1NoAve,FALSE);
  END;    {of SetUpMenus}


PROCEDURE GetSize(VAR lresult: INTEGER);
{puts up a dialog to get the new size of the field.}
VAR tDialog: DialogPtr;
    theItem: INTEGER;
    tHandle: Handle;
    dummy: INTEGER;
 
BEGIN
  tDialog := GetNewDialog(257,NIL,POINTER(-1));
  ModalDialog(NIL,theItem);
  IF theItem = 1 THEN
    BEGIN
    GetDItem(tDialog,4,dummy,tHandle,tempRect);
    GetIText(tHandle,tempStr);
    IF LENGTH(tempStr) = 0 THEN
      lInteger := 0
    ELSE
      StringToNum(tempStr,lInteger);
    lresult := LoWord(linteger);
    END;
  DisposDialog(tDialog);
  IF (lresult < 2) OR (lresult > MaxN) THEN
    lresult := 10;
END;

PROCEDURE GetTemp(VAR lresult: REAL);
{puts up a dialog to get the new temperature of the field.}
VAR tDialog: DialogPtr;
    theItem: INTEGER;
    tHandle: Handle;
    dummy: INTEGER;
 
BEGIN
  tDialog := GetNewDialog(258,NIL,POINTER(-1));
  ModalDialog(NIL,theItem);
  IF theItem = 1 THEN
    BEGIN
    GetDItem(tDialog,4,dummy,tHandle,tempRect);
    GetIText(tHandle,tempStr);
    IF LENGTH(tempStr) <> 0 THEN
      BEGIN
      Rewrite(tFile);
      Writeln(tFile,tempStr);
      Reset(tFile);
      Readln(tFile,lResult);
      END;
    END;
  DisposDialog(tDialog);
END;

PROCEDURE TellSpeed;
{puts up a dialog to tell the user to type a number to change the speed.}
VAR tDialog: DialogPtr;
    theItem: INTEGER;
    tHandle: Handle;
    dummy: INTEGER;
 
BEGIN
  tDialog := GetNewDialog(259,NIL,POINTER(-1));
  ModalDialog(NIL,theItem);
  DisposDialog(tDialog);
END;

PROCEDURE doAboutBox;
{puts up a the About IsingGraf dialog box.}
VAR tDialog: DialogPtr;
    theItem: INTEGER;
    tHandle: Handle;
    dummy: INTEGER;
 
BEGIN
  tDialog := GetNewDialog(260,NIL,POINTER(-1));
  ModalDialog(NIL,theItem);
  DisposDialog(tDialog);
END;

 
{----------------------------------------------------------------------------------}
PROCEDURE DoCommand (mResult: LONGINT);
{ Execute command specified by mResult, the result of MenuSelect }
 
  VAR theItem: INTEGER;  {menu item number from mResult low-order word}
      theMenu: INTEGER;  {menu number from mResult high-order word}
      name: Str255;      {desk accessory name}
      temp: INTEGER;
      savePort: GrafPtr; {temporary save of grafport}
      i,j,n1: INTEGER;
 
 
  BEGIN {PROCEDURE DoCommand}
  theItem := LoWord(mResult);
  theMenu := HiWord(mResult);             {set menu item number and menu number}
 
  CASE theMenu OF                         {case on menu ID}
 
  appleID:
    BEGIN
    IF theItem = 1 THEN
      doAboutBox
    ELSE
      BEGIN
      GetItem(myMenus[appleM],theItem,name);  {get name of desk accesory}
      GetPort(SavePort);
      temp := OpenDeskAcc(name);              {open desk accesory}
      SetPort(SavePort);
      END
    END;    {of appleID}
 
  fileID:         {quit}
     BEGIN
     Close(tFile);
     ExitToShell;        {quit}
     END;
 
  editID:
    BEGIN                         {call Desk Manager to handle editing command if }
    IF NOT SystemEdit(theItem-1) THEN  {desk accessory window is the active window}
                                       {application window is the active window}
      BEGIN
      CASE theItem OF
	undoCommand:
	  BEGIN
	  END;
	cutCommand:
	  BEGIN
	  END;
	copyCommand:
	  BEGIN
	  END;
	pasteCommand:
	  BEGIN
	  END;
	clearCommand:
	  BEGIN
	  END;
	cSpecialCommand:
	  BEGIN
	  END;
        END;
      END
    END;
 
  Menu1ID:  {control menu}
    CASE theItem OF
    M1NewSize: {New Field Size Item}
         BEGIN
	 GetSize(Np1);    {put up dialog for new size}
	 N := Np1-1;
	 Nm1 := N - 1;
	 NSpins := Np1 * Np1; {Set the dimension variables}
	 
	 Spacing := 250 DIV Np1;
	 IF Spacing < 1 THEN
	   Spacing := 1;
	 hOffset := 5;
	 vOffset := 5;  {Set the plotting variables to defaults}

	 tComp := 1;
	 tComp := tComp*Np1*Np1*2*Np1*Np1*2*FewSteps;
	 IF tComp > 2000000000 THEN
	   VeryBig := TRUE
	 ELSE
	   VeryBig := FALSE;
		 
	 RandField;	 
	 SetValues;

	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M1NewTemp:
         BEGIN {New Temperature Item}
	 GetTemp(kToJ); {put up dialog for new temp}
         MakeLookup(kToJ);
	 SetValues;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M1YesAve:
         BEGIN {Display Averages Item}
	 Averaging := TRUE;
	 SetValues;
	 CheckItem(myMenus[FirstM],M1YesAve,TRUE);
	 CheckItem(myMenus[FirstM],M1NoAve,FALSE);
         END;
    M1NoAve:
         BEGIN {Don't Display Averages Item}
	 Averaging := FALSE;
	 CheckItem(myMenus[FirstM],M1YesAve,FALSE);
	 CheckItem(myMenus[FirstM],M1NoAve,TRUE);
         END;
    M1NewAve:
         BEGIN {Reset Averages Item}
	 IF Averaging = TRUE THEN
	   SetValues;
         END;
    M1Speed:
         BEGIN  {speed item}
	 TellSpeed;
	 END;
      END; {case of Menu1ID}

  Menu2ID: {Setup Menu}
    CASE theItem OF
    M2Item1: {Random Field Item}
         BEGIN
	 RandField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M2Item2: {Checkboard Field Item}
         BEGIN
	 CheckField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M2Item3: {Interface Field Item}
         BEGIN
	 IntfField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M2Item4: {Big Spot Field Item}
         BEGIN
	 SpotField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M2Item5: {Ring Field Item}
         BEGIN
	 RingField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M2Item6: {X Field Item}
         BEGIN
	 XField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M2Item7: {Lined Field Item}
         BEGIN
	 LinedField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
    M2Item8: {B Field Item}
         BEGIN
	 BField;
	 InvalRect(GenWind^.portRect);
	 doUpdate;
         END;
      END;
 
  END;    {of menu case}          {to indicate completion of command, call }
  HiliteMenu(0);                  {Menu Manager to unhighlight menu title  }
                                  {(highlighted by MenuSelect)             }
  END;  {of DoCommand}
 
 
 
{----------------------------------------------------------------------------------}
PROCEDURE MainLoop;
 
  BEGIN
 
  SystemTask;                             {do desk accessories}
 
  IF GetNextEvent(everyEvent,myEvent) {call Toolbox Event Manager to get the next }
    THEN                              { event that the application should handle}
      CASE myEvent.what OF            {case on event type}
 
      mouseDown:            {mouse button down:  call Window Manager to learn where}
        CASE FindWindow(myEvent.where,whichWindow) OF
 
        inSysWindow:        {desk accessory window: call Desk Manager to handle it}
          SystemClick(myEvent,whichWindow);
 
        inMenuBar:          {menu bar:  call Menu Manager to learn which command; }
          DoCommand(MenuSelect(myEvent.where)); { then execute it}
 
        inContent:
          BEGIN
          IF whichWindow <> FrontWindow THEN
            SelectWindow(whichWindow)
          ELSE
            BEGIN
            END;
          END     {of inContent}
 
        END;    {of mouseDown}
 
     keyDown, autoKey:              {key pressed once or held down to repeat}
        BEGIN
        theChar := CHR(BitAnd(myEvent.message,charCodeMask));  {get the character}
        IF BitAnd(myEvent.modifiers,cmdKey) <> 0  THEN { if Command key down,    }
          DoCommand(MenuKey(theChar))                  { execute it              }
        ELSE
          BEGIN
	  IF (theChar >= '0') AND (theChar <= '9') THEN
	    BEGIN
	    Speed := Ord('9') - Ord(theChar);
	    Speed := BitSL(1,Speed);
	    IF Speed = 1 THEN
	      FewSteps := 400
	    ELSE
	      BEGIN
	      FewSteps := 50 DIV Speed;
	      IF FewSteps < 1 THEN
	        FewSteps := 1;
	      END
	    END;
          END;
        END;
 
      activateEvt:
      BEGIN
      IF BitAnd(myEvent.modifiers,activeFlag) <> 0 THEN   {window becoming active}
        IF myEvent.message = Ord4(GenWind) THEN
          BEGIN
          SelectWindow(GenWind);
          SetPort(GenWind);
          END
        ELSE
          BEGIN                  {application window is becoming inactive: }
          END;
      END;   {of activateEvt}
 
      updateEvt:
        BEGIN
	doUpdate;
        END
 
     END;    {of event case}
 
END; {main loop}
 
 
 
 
BEGIN  {Kgraph}
{ Initialization }
InitGraf(@thePort);           {initialize QuickDraw}
InitFonts;                    {initialize Font Manager}
FlushEvents(everyEvent,0);    {call OS Event Manager to discard any previous events}
InitWindows;                  {initialize Window Manager}
InitMenus;                    {initialize Menu Manager}
TEInit;                       {initialize TextEdit}
InitDialogs(NIL);             {initialize Dialog Manager}
InitCursor;                   {call QuickDraw to make cursor (pointer) an arrow}
 
SetUpMenus;                   {set up menus and menu bar}
 
WITH screenBits.bounds DO     {call QuickDraw to set dragging boundaries; ensure at }
  SetRect(dragRect,4,24,right-4,bottom-4); { least 4 by 4 pixels will remain visible}
 
GenWind := GetNewWindow(windowID,@GenWRecord,POINTER(-1)); {put up window}
 
hCurs := GetCursor (iBeamCursor);
iBeam := hCurs^^;                              {get I-beam cursor}
 
SelectWindow(GenWind);
SetPort(GenWind);

Open(tFile,tempName);   {open the stupid text file}
 

 

 lInteger := SizeOf(RowType);
 FOR i := 0 TO MaxN DO
   Field[i] := RowPtr(NewPtr(lInteger));   {allocate the field}
 
 N := 10;
 Nm1 := N - 1;
 Np1 := N + 1;
 NSpins := Np1 * Np1;   {Set the dimension variables to starting defaults}
 
 Speed := 1;
 FewSteps := 400;
 
 Spacing := 250 DIV Np1;
 IF Spacing < 1 THEN
   Spacing := 1;
 hOffset := 5;
 vOffset := 5;          {Set the plotting variables to starting defaults}

 tComp := 1;
 tComp := tComp*Np1*Np1*2*Np1*Np1*2*FewSteps;
 IF tComp > 2000000000 THEN
   VeryBig := TRUE
 ELSE
   VeryBig := FALSE;

 kToJ := 2.3;    {set default temperature}
 MakeLookup(kToJ);

 RandField;
 WalkField;
 
 SetValues;
 
 
REPEAT
  IF Averaging THEN
    BEGIN
    REPEAT
      IF VeryBig THEN
        MonteCarloR(FewSteps, tEAve, tESqAve, tSpinAve)
      ELSE
        MonteCarloI(FewSteps, tEAve, tESqAve, tSpinAve);
      invNSteps := 1/(NSteps + FewSteps);
      EAve := (EAve*NSteps + tEAve*FewSteps)*invNSteps;
      ESqAve := (ESqAve*NSteps + tESqAve*FewSteps)*invNSteps;
      SpinAve := (SpinAve*NSteps + tSpinAve*FewSteps)*invNSteps;
      NSteps := NSteps + FewSteps;
      ShowAverages;
    UNTIL EventAvail(everyEvent,myEvent);
    END
  ELSE
    BEGIN
    REPEAT
      eqMonteCarlo(FewSteps);
      NSteps := NSteps + FewSteps;
    UNTIL EventAvail(everyEvent,myEvent);
    END;

  mainloop;
UNTIL TRUE=FALSE;
END.
